// //////////////////////////////////////////////////////////////////////////
// Implementation file ConflictFileLoader.cpp for class ConflictFileLoader
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/06/16
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "ConflictFileLoader.h"
#include "LineReader.h"
#include <memory>

std::string ConflictFileLoader::ms_conflictLimiter( ">>>>>>>" );

std::string ConflictFileLoader::ms_conflictSeparator( "=======" );


ConflictFileLoader::ConflictFileLoader( CString fileName, 
                                  ConflictTextBuilder *pTargetBuffer,
                                  ConflictTextBuilder *pMergedBuffer ) :
    m_fileName( fileName ),
    m_pTargetBuffer( pTargetBuffer ),
    m_pMergedBuffer( pMergedBuffer )
{
}


ConflictFileLoader::~ConflictFileLoader()
{
}


bool 
ConflictFileLoader::loadFile()
{
  try 
  {
    CFile file( m_fileName, CFile::modeRead | CFile::shareDenyNone );

    std::auto_ptr<LineReader> ptrLineReader( 
        LineReader::makeLineReaderFor( &file ) );

    readLines( ptrLineReader.get() );
  }
  catch ( CFileException *e )
  {
    e->Delete();
    return false;
  }

  return true;
}


void 
ConflictFileLoader::readLines( LineReader *lineReader )
{
  try
  {
    do
    {
      std::string line( lineReader->readLine() );
      if ( isLineStartingConflictBlock( line ) )
        readConflictBlock( line, lineReader );
      else
        appendLineToTextBuffers( line );
    }
    while ( true );
  }
  catch( LineReader::NoMoreLine e )
  {
  }
}


bool 
ConflictFileLoader::isLineStartingConflictBlock( std::string line )
{
  static const std::string conflictHeading( "<<<<<<<" );
  return line.compare( 0, conflictHeading.length(), conflictHeading ) == 0;
}


void 
ConflictFileLoader::appendLineToTextBuffers( std::string line )
{
  m_pTargetBuffer->appendLine( line );
  m_pMergedBuffer->appendLine( line );
}


void 
ConflictFileLoader::readConflictBlock( std::string headLine,
                                       LineReader *lineReader )
{
  readTargetLines( lineReader );
  readMergedLines( lineReader );
}


void 
ConflictFileLoader::readTargetLines( LineReader *lineReader )
{
  try
  {
    int separatorPos;
    m_pTargetBuffer->beginConflictBlock();
    do
    {
      std::string line( lineReader->readLine() );
      separatorPos = line.find( ms_conflictSeparator );
      if ( separatorPos >= 0 )
        line.resize( separatorPos );
      m_pTargetBuffer->appendLine( line );
    }
    while ( separatorPos < 0 );
  }
  catch ( LineReader::NoMoreLine e )
  {
  }
  m_pTargetBuffer->endConflictBlock();
}

void 
ConflictFileLoader::readMergedLines( LineReader *lineReader )
{
  try
  {
    int nLimiterPos;
    m_pMergedBuffer->beginConflictBlock();
    do
    {
      std::string line( lineReader->readLine() );
      nLimiterPos = line.find( ms_conflictLimiter );
      if ( nLimiterPos >= 0 )
      {
        extractMergedRevision( nLimiterPos, line );
        line.resize( nLimiterPos );
      }
      m_pMergedBuffer->appendLine( line );
    }
    while ( nLimiterPos < 0 );
  }
  catch ( LineReader::NoMoreLine e )
  {
  }
  m_pMergedBuffer->endConflictBlock();
}


void 
ConflictFileLoader::extractMergedRevision( int nLimiterPos, 
                                           std::string line )
{
  int nRevisionPos = nLimiterPos + ms_conflictLimiter.length() +1;
  m_mergedRevision = line.substr( nRevisionPos );
}


std::string 
ConflictFileLoader::mergedRevision()
{
  return m_mergedRevision;
}
